#!/usr/bin/env python

from narwhal import conf
from narwhal import valve
from narwhal import pathutil
from narwhal.download_repose import ReposeMavenConnector
import unittest2 as unittest
import requests
import os.path
import argparse
import logging


logger = logging.getLogger(__name__)


def setUpModule():
    pathutil.create_folder('test/bin')
    pathutil.create_folder('test/conf')
    pathutil.create_folder('test/log')
    pathutil.create_folder('test/var')
    #pathutil.clear_folder('test/bin')
    pathutil.clear_folder('test/conf')
    pathutil.clear_folder('test/log')
    pathutil.clear_folder('test/var')
    if not os.path.exists('test/bin/repose-valve.jar'):
        rmc = ReposeMavenConnector()
        logger.debug('Downloading valve jar')
        rmc.get_repose(valve_dest='test/bin/repose-valve.jar',
                       get_filter=False, get_ext_filter=False)
    if not os.path.exists('test/bin/filter-bundle.ear'):
        rmc = ReposeMavenConnector()
        logger.debug('Downloading filter bundle')
        rmc.get_repose(filter_dest='test/bin/filter-bundle.ear',
                       get_valve=False, get_ext_filter=False)
    logger.debug('module setup complete')


class TestValveWaitOnStart(unittest.TestCase):
    def setUp(self):
        self.valve = None
        logger.debug('Setting config files')
        conf.process_folder_contents(folder='test-configs',
                                     dest_path='test/conf',
                                     params={'port': 12345})

    def tearDown(self):
        if self.valve is not None:
            logger.debug('shutting down valve')
            self.valve.stop()

    def test_wait_on_start(self):
        logger.debug('starting valve')
        self.valve = valve.Valve(config_dir='test/conf', port=12345,
                                 stop_port=6789, wait_on_start=True,
                                 jar_file='test/bin/repose-valve.jar')
        logger.debug('valve started, checking port')
        resp = requests.get('http://localhost:12345/')
        self.assertEqual(resp.status_code, 200)
        logger.debug('good to go')


class TestValveArgs(unittest.TestCase):
    def test_config_dir(self):
        pargs = valve.Valve.construct_args(config_dir='asdf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'asdf', 'start'])

        pargs = valve.Valve.construct_args(config_dir='path/to/configs',
                                           port=None, https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'path/to/configs', 'start'])

        pargs = valve.Valve.construct_args(config_dir='path with spaces',
                                           port=None, https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'path with spaces', 'start'])

    def test_port(self):
        pass

    def test_https_port(self):
        pass

    def test_jar_file(self):
        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', 'start'])

        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file='path/to/file.jar',
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', 'path/to/file.jar',
                                 '-c', 'conf', 'start'])

        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file='filename with spaces.jar',
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', 'filename with spaces.jar',
                                 '-c', 'conf', 'start'])

    def test_stop_port(self):
        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', 'start'])

        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=7777, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', '-s', '7777', 'start'])

        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port='8888', insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', '-s', '8888', 'start'])

    def test_insecure(self):
        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', 'start'])

        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=True,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', '-k', 'start'])

    def test_conn_fw(self):
        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw=None)
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', 'start'])

        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw='jersey')
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', '-cf', 'jersey', 'start'])

        pargs = valve.Valve.construct_args(config_dir='conf', port=None,
                                           https_port=None,
                                           jar_file=valve._default_jar_file,
                                           stop_port=None, insecure=False,
                                           conn_fw='apache')
        self.assertEqual(pargs, ['java', '-jar', valve._default_jar_file,
                                 '-c', 'conf', '-cf', 'apache', 'start'])


def run():
    parser = argparse.ArgumentParser()
    parser.add_argument('--print-log', help="Print the log to STDERR.",
                        action='store_true')
    args = parser.parse_args()

    if args.print_log:
        logging.basicConfig(level=logging.DEBUG,
                            format=('%(asctime)s %(levelname)s:%(name)s:'
                                    '%(funcName)s:'
                                    '%(filename)s(%(lineno)d):'
                                    '%(threadName)s(%(thread)d):%(message)s'))

    unittest.main(argv=[''])


if __name__ == '__main__':
    run()



#!/usr/bin/env python

# Copyright (C) 2012-2013 Richard Sartor <richard.sartor@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import subprocess
import socket
import logging
import threading
import Queue
import time
import requests

from . import __version__


logger = logging.getLogger(__name__)


_default_jar_file = 'usr/share/repose/repose-valve.jar'


class Valve:
    def __init__(self, config_dir, port=None, https_port=None, jar_file=None,
                 stop_port=None, insecure=False, wait_on_start=False,
                 wait_timeout=None, conn_fw=None):
        logger.debug('Creating new Valve object (config_dir=%s, '
                     'jar_file=%s, stop_port=%s, insecure=%s)' %
                     (config_dir, jar_file, stop_port, insecure))

        if jar_file is None:
            jar_file = _default_jar_file

        if stop_port is None:
            if port is None:
                stop_port = 9090
            else:
                stop_port = port + 1000

        if wait_on_start and port is None and https_port is None:
            raise ValueError("Either 'port' and/or 'https_port' must specify "
                             "a port number if 'wait_on_start' is True")

        if conn_fw is not None:
            fws = ['jersey', 'apache']
            if conn_fw not in fws:
                raise ValueError('"%s" is not a valid connection framework. '
                                 'If sepecified, conn_fw must be one of: '
                                 '[%s]' % conn_fw, ', '.join(fws))

        pargs = self.construct_args(config_dir=config_dir, port=port,
                                    https_port=https_port, jar_file=jar_file,
                                    stop_port=stop_port, insecure=insecure,
                                    conn_fw=conn_fw)

        self.config_dir = config_dir
        self.port = port
        self.https_port = https_port
        self.jar_file = jar_file
        self.stop_port = stop_port
        self.insecure = insecure

        logger.debug('Starting valve with the following command line: "%s"' %
                     ' '.join(pargs))

        self.proc = subprocess.Popen(pargs, stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
        self.stdout = ThreadedStreamReader(self.proc.stdout)
        self.stderr = ThreadedStreamReader(self.proc.stderr)

        if wait_on_start:
            self.wait_for_node_to_start(wait_timeout=wait_timeout)

        logger.debug('New Valve object initialized (pid=%i)' %
                     self.proc.pid)

    @staticmethod
    def construct_args(config_dir, port, https_port, jar_file, stop_port,
                       insecure, conn_fw):
        """Take the provided parameters and turn them into the command line
        arguments to invoke Valve."""

        pargs = [
            'java', '-jar', jar_file,
            '-c', config_dir,
        ]

        if stop_port is not None:
            pargs.append('-s')
            pargs.append(str(stop_port))

        if port is not None:
            pargs.append('-p')
            pargs.append(str(port))

        if https_port is not None:
            pargs.append('-ps')
            pargs.append(str(https_port))

        if insecure:
            pargs.append('-k')

        if conn_fw is not None:
            pargs.append('-cf')
            pargs.append(conn_fw)

        pargs.append('start')

        return pargs

    def wait_for_node_to_start(self, wait_timeout=None):
        if self.port is None and self.https_port is None:
            raise ValueError("No valid port was set. Cannot wait on the node.")

        if self.port is not None:
            wait_url = 'http://localhost:%s/' % str(self.port)
        else:
            wait_url = 'https://localhost:%s' % str(self.https_port)

        wait_for_node_to_start(url=wait_url, wait_timeout=wait_timeout)

    def stop(self, wait=True):
        try:
            logger.debug('Shutting down stdout and stderr readers.')
            self.stdout.shutdown()
            self.stderr.shutdown()

            logger.debug('Attempting to stop Valve object (pid=%i, '
                         'stop_port=%s)' % (self.proc.pid, self.stop_port))
            s = socket.create_connection(('localhost', self.stop_port))
            s.send('stop\r\n')
            if wait:
                logger.debug('Waiting for process to end (pid=%i)' %
                             self.proc.pid)
                self.wait(timeout=30)
                if self.proc.poll() is None:
                    # it timed out while waiting
                    raise Exception
        except:
            logger.debug('Couldn\'t stop using the stop port, killing instead '
                         '(pid=%i)' % self.proc.pid)
            self.proc.kill()
        logger.debug('Valve stopped (pid=%i)' % self.proc.pid)

    def wait(self, timeout=30):
        t1 = time.time()
        while True:
            logger.debug('polling')
            if self.proc.poll() is not None:
                logger.debug('child process stopped')
                return
            t2 = time.time()
            if timeout is not None and t2 - t1 > timeout:
                logger.debug('timed out')
                return
            time.sleep(1)


def wait_for_node_to_start(url=None, scheme='http', port=None,
                           wait_timeout=None):
    if url is None:
        if port is None:
            raise ArgumentError("Must specify either url or port "
                                "parameter.")
        url = '%s://localhost:%s' % (scheme, port)

    t1 = time.time()
    while True:
        try:
            resp = requests.get(url)
            if int(resp.status_code) != 500:
                # if it's not a 500 error, then it's done starting
                break
        except:
            pass
        time.sleep(1)
        t2 = time.time()
        if wait_timeout is not None and t2 - t1 > wait_timeout:
            logger.debug('wait_on_start timed out')
            break


class ThreadedStreamReader:
    def __init__(self, stream):
        self.stream = stream
        self.queue = Queue.Queue()
        self._shutdown = False
        self.thread = threading.Thread(target=self._thread_target)
        self.thread.daemon = True
        self.thread.start()

    def _thread_target(self):
        for line in self.stream.xreadlines():
            if self._shutdown:
                break
            self.queue.put(line)

    def readline(self, timeout=None):
        s = self.queue.get(timeout=timeout)
        self.queue.task_done()
        return s

    def readlines(self):
        lines = []
        while not self.queue.empty():
            lines.append(self.readline())
        return lines

    def shutdown(self):
        self._shutdown = True


def stream_printer(fin, fout):
    while True:
        for line in fin.readlines():
            fout.write(line)
            fout.flush()
        time.sleep(1)



#!/usr/bin/env python

import requests

__version_info__ = (0, 7)
__version__ = '.'.join(map(str, __version_info__))


port_base = 8080
port_port = None


def get_next_open_port(start=None):
    global port_port
    if start is not None:
        port_port = start
    elif port_port is None:
        port_port = port_base

    while port_port < 65535:
        try:
            requests.get('http://localhost:%i' % port_port)
        except requests.exceptions.ConnectionError as e:
            port_port += 1
            return port_port - 1
        except:
            pass
        port_port += 1
    raise Exception('Ran out of ports')



#!/usr/bin/env python

from setuptools import setup
import narwhal

setup(
    name='narwhal',
    version=narwhal.__version__,
    packages=['narwhal'],
    license=open('LICENSE').read(),
    long_description=open('README.rst').read() + '\n\n' +
                     open('HISTORY.rst').read(),
    author='Richard Sartor',
    author_email='richard.sartor@rackspace.com',
    url='https://github.com/richard-sartor/narwhal',
    description='Setup scripts for repose',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
    install_requires=[
        'requests',
        'argparse',
    ],
    entry_points={
        'console_scripts': [
            'download-repose = narwhal.download_repose:run',
            'run-repose = narwhal.run_repose:run',
            'configure-repose = narwhal.configure_repose:run',
        ],
    }
)
